@@ -1,4 +1,4 @@
-# -*- test-case-name: axiom.test.test_attributes -*-
+# -*- test-case-name: axiom.test.test_attributes,axiom.test.test_reference -*-
 
 import os
 
@@ -11,6 +11,7 @@ from zope.interface import implements
 
 from twisted.python import filepath
 from twisted.python.components import registerAdapter
+from twisted.python.reflect import qual
 
 from epsilon.extime import Time
 
@@ -20,8 +21,6 @@ from axiom.errors import NoCrossStoreReferences, BrokenReference
 
 from axiom.iaxiom import IComparison, IOrdering, IColumn, IQuery
 
-USING_APSW = False
-
 _NEEDS_FETCH = object()         # token indicating that a value was not found
 
 __metaclass__ = type
@@ -411,12 +410,12 @@ class SQLAttribute(inmemory, Comparable):
     _type = None
     type = property(*type())
 
-    def __get__(self, oself, type=None):
-        if type is not None and oself is None:
+    def __get__(self, oself, cls=None):
+        if cls is not None and oself is None:
             if self._type is not None:
-                assert self._type == type
+                assert self._type == cls
             else:
-                self._type = type
+                self._type = cls
             return self
 
         pyval = getattr(oself, self.underlying, _NEEDS_FETCH)
@@ -448,12 +447,24 @@ class SQLAttribute(inmemory, Comparable):
                 # would be worthwhile.
 
                 return self.default
-            # cache python value
             pyval = self.outfilter(dbval, oself)
+            # An upgrader may have changed the value of this attribute.  If so,
+            # return the new value, not the old one.
+            if dbval != getattr(oself, self.dbunderlying):
+                return self.__get__(oself, cls)
+            # cache python value
             setattr(oself, self.underlying, pyval)
         return pyval
 
     def loaded(self, oself, dbval):
+        """
+        This method is invoked when the item is loaded from the database, and
+        when a transaction is reverted which restores this attribute's value.
+
+        @param oself: an instance of an item which has this attribute.
+
+        @param dbval: the underlying database value which was retrieved.
+        """
         setattr(oself, self.dbunderlying, dbval)
         delattr(oself, self.underlying) # member_descriptors don't raise
                                         # attribute errors; what gives?  good
@@ -461,7 +472,9 @@ class SQLAttribute(inmemory, Comparable):
 
     def _convertPyval(self, oself, pyval):
         """
-        Convert a Python value to a value suitable for inserting into the database.
+        Convert a Python value to a value suitable for inserting into the
+        database.
+
         @param oself: The object on which this descriptor is an attribute.
         @param pyval: The value to be converted.
         @return: A value legal for this column in the database.
@@ -480,6 +493,7 @@ class SQLAttribute(inmemory, Comparable):
         oself.__dirty__[self.attrname] = self, dbval
         oself.touch()
         setattr(oself, self.underlying, pyval)
+        setattr(oself, self.dbunderlying, dbval)
         if st is not None and st.autocommit:
             st._rejectChanges += 1
             try:
@@ -927,14 +941,10 @@ class text(SQLAttribute):
                 self, "unicode string without NULL bytes", pyval)
         return pyval
 
-    if USING_APSW:
-        def outfilter(self, dbval, oself):
-            if type(dbval) is str:
-                return unicode(dbval, 'ascii')
-            return dbval
-    else:
-        def outfilter(self, dbval, oself):
-            return dbval
+    def outfilter(self, dbval, oself):
+        return dbval
+
+
 
 class textlist(text):
     delimiter = u'\u001f'
@@ -1084,8 +1094,14 @@ class reference(integer):
             return 'None'
         return 'reference(%d)' % (sid,)
 
-    def __get__(self, oself, type=None):
-        rv = super(reference, self).__get__(oself, type)
+
+    def __get__(self, oself, cls=None):
+        """
+        Override L{integer.__get__} to verify that the value to be returned is
+        currently a valid item in the same store, and to make sure that legacy
+        items are upgraded if they happen to have been cached.
+        """
+        rv = super(reference, self).__get__(oself, cls)
         if rv is self:
             # If it's an attr lookup on the class, just do that.
             return self
@@ -1100,6 +1116,9 @@ class reference(integer):
             assert self.whenDeleted is reference.NULLIFY, (
                 "not sure what to do if not...")
             return None
+        if rv.__legacy__:
+            delattr(oself, self.underlying)
+            return super(reference, self).__get__(oself, cls)
         return rv
 
     def prepareInsert(self, oself, store):
@@ -1132,6 +1151,20 @@ class reference(integer):
 
         referee = oself.store.getItemByID(dbval, default=None, autoUpgrade=not oself.__legacy__)
         if referee is None and self.whenDeleted is not reference.NULLIFY:
+
+            # If referee merely changed to another valid referent,
+            # SQLAttribute.__get__ will notice that what we returned is
+            # inconsistent and try again.  However, it doesn't know about the
+            # BrokenReference that is raised if the old referee is no longer a
+            # valid referent.  Check to see if the dbunderlying is still the
+            # same as the dbval passed in.  If it's different, we should try to
+            # load the value again.  Only if it is unchanged will we raise the
+            # BrokenReference.  It would be better if all of this
+            # change-detection logic were in one place, but I can't figure out
+            # how to do that. -exarkun
+            if dbval != getattr(oself, self.dbunderlying):
+                return self.__get__(oself, None)
+
             raise BrokenReference('Reference to storeID %r is broken' % (dbval,))
         return referee
 
@@ -1209,19 +1242,19 @@ class AbstractFixedPointDecimal(integer):
 
     def __init__(self, **kw):
         integer.__init__(self, **kw)
-        self.exponent = (10 ** self.decimalPlaces)
 
 
     def infilter(self, pyval, oself, store):
         if pyval is None:
             return None
-        elif isinstance(pyval, Decimal):
+        if isinstance(pyval, (int, long)):
+            pyval = Decimal(pyval)
+        if isinstance(pyval, Decimal):
+            # Python < 2.5.2 compatibility:
+            # Use to_integral instead of to_integral_value.
+            dbval = int((pyval * 10**self.decimalPlaces).to_integral())
             return super(AbstractFixedPointDecimal, self).infilter(
-                int(pyval._rescale(-(self.decimalPlaces)) * self.exponent),
-                oself, store)
-        elif isinstance(pyval, (int, long)):
-            return super(AbstractFixedPointDecimal, self).infilter(
-                pyval * self.exponent, oself, store)
+                dbval, oself, store)
         else:
             raise TypeError(
                 "attribute [%s.%s = AbstractFixedPointDecimal(...)] must be "
@@ -1232,7 +1265,7 @@ class AbstractFixedPointDecimal(integer):
     def outfilter(self, dbval, oself):
         if dbval is None:
             return None
-        return Decimal(int(dbval)) / self.exponent
+        return Decimal(dbval) / 10**self.decimalPlaces
 
 
     def compare(self, other, sqlop):
